@@ -1,19 +1,19 @@
 /*
- *   2003 Jrg Bsner <ich@joerg-boesner.de>
+ *   Copyright 2003 Jrg Bsner <ich@joerg-boesner.de>
  *     patch for switching the touchpad off (for example, when a
  *     USB mouse is connected)
  *
- *   2003 Hartwig Felger <hgfelger@hgfelger.de>
+ *   Copyright 2003 Hartwig Felger <hgfelger@hgfelger.de>
  *     patch to make the horizontal wheel replacement buttons work.
  *
- *   2002 Peter Osterlund <petero2@telia.com>
+ *   Copyright 2002 Peter Osterlund <petero2@telia.com>
  *     patches for fast scrolling, palm detection, edge motion,
  *     horizontal scrolling
  *
- *   2002 S. Lehner <sam_x@bluemail.ch>
+ *   Copyright 2002 S. Lehner <sam_x@bluemail.ch>
  *     for newer Firmware (5.8) protocol changes for 3rd to 6th button
  *
- *	 Copyright (C) 2001 Stefan Gmeiner <riddlebox@freesurf.ch>
+ *   Copyright (C) 2001 Stefan Gmeiner <riddlebox@freesurf.ch>
  *     start merging tpconfig and gpm code to an xfree input module
  *     adding some changes and extensions (ex. 3rd and 4th button)
  *
@@ -107,7 +107,7 @@ typedef enum {
 #define DEV_INPUT_EVENT "/dev/input"
 #define EVENT_DEV_NAME "event"
 
-#define VERSION "0.12.0"
+#define VERSION "0.12.1"
 
 /*****************************************************************************
  * Forward declaration
@@ -163,7 +163,7 @@ static void
 SetDeviceAndProtocol(LocalDevicePtr local)
 {
     char *str_par;
-    SynapticsPrivatePtr priv = local->private;
+    SynapticsPrivate *priv = local->private;
 
     priv->proto = SYN_PROTO_PSAUX;
     str_par = xf86FindOptionValue(local->options, "Protocol");
@@ -215,7 +215,7 @@ static InputInfoPtr
 SynapticsPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
 {
     LocalDevicePtr local;
-    SynapticsPrivatePtr priv;
+    SynapticsPrivate *priv;
 #ifdef XFREE_4_0_3
     XF86OptionPtr optList;
 #else
@@ -226,7 +226,7 @@ SynapticsPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     unsigned long now;
 
     /* allocate memory for SynaticsPrivateRec */
-    priv = xcalloc (1, sizeof (SynapticsPrivateRec));
+    priv = xcalloc (1, sizeof (SynapticsPrivate));
     if (!priv)
 	return NULL;
 
@@ -446,7 +446,7 @@ DeviceControl (DeviceIntPtr dev, int mode)
 	{
 	    int shmid;
 	    LocalDevicePtr local = (LocalDevicePtr) dev->public.devicePrivate;
-	    SynapticsPrivatePtr priv = (SynapticsPrivatePtr) (local->private);
+	    SynapticsPrivate *priv = (SynapticsPrivate *) (local->private);
 	    RetValue = DeviceOff( dev );
 	    if (priv->shm_config)
 		if ((shmid = xf86shmget(SHM_SYNAPTICS, 0, 0)) != -1)
@@ -464,7 +464,7 @@ static Bool
 DeviceOn (DeviceIntPtr dev)
 {
     LocalDevicePtr local = (LocalDevicePtr) dev->public.devicePrivate;
-    SynapticsPrivatePtr priv = (SynapticsPrivatePtr) (local->private);
+    SynapticsPrivate *priv = (SynapticsPrivate *) (local->private);
 
     DBG(3, ErrorF("Synaptics DeviceOn called\n"));
 
@@ -505,7 +505,7 @@ static Bool
 DeviceOff(DeviceIntPtr dev)
 {
     LocalDevicePtr local = (LocalDevicePtr) dev->public.devicePrivate;
-    SynapticsPrivatePtr priv = (SynapticsPrivatePtr) (local->private);
+    SynapticsPrivate *priv = (SynapticsPrivate *) (local->private);
 
     DBG(3, ErrorF("Synaptics DeviceOff called\n"));
 
@@ -557,7 +557,7 @@ move_distance(int dx, int dy)
 }
 
 static edge_type
-edge_detection(SynapticsPrivatePtr priv, int x, int y)
+edge_detection(SynapticsPrivate *priv, int x, int y)
 {
     edge_type edge = 0;
 
@@ -578,7 +578,7 @@ static CARD32
 timerFunc(OsTimerPtr timer, CARD32 now, pointer arg)
 {
     LocalDevicePtr local = (LocalDevicePtr) (arg);
-    SynapticsPrivatePtr priv = (SynapticsPrivatePtr) (local->private);
+    SynapticsPrivate *priv = (SynapticsPrivate *) (local->private);
     struct SynapticsHwState hw;
     int delay;
     int sigstate;
@@ -626,7 +626,7 @@ static int clamp(int val, int min, int max)
 static void
 ReadInput(LocalDevicePtr local)
 {
-    SynapticsPrivatePtr priv = (SynapticsPrivatePtr) (local->private);
+    SynapticsPrivate *priv = (SynapticsPrivate *) (local->private);
     struct SynapticsHwState hw;
     int delay = 0;
     Bool newDelay = FALSE;
@@ -642,9 +642,9 @@ ReadInput(LocalDevicePtr local)
 }
 
 static int
-HandleMidButtonEmulation(SynapticsPrivatePtr priv, struct SynapticsHwState* hw, long* delay)
+HandleMidButtonEmulation(SynapticsPrivate *priv, struct SynapticsHwState *hw, long *delay)
 {
-    SynapticsSHMPtr para = priv->synpara;
+    SynapticsSHM *para = priv->synpara;
     Bool done = FALSE;
     long timeleft;
     int mid = 0;
@@ -714,9 +714,9 @@ HandleMidButtonEmulation(SynapticsPrivatePtr priv, struct SynapticsHwState* hw,
 }
 
 static int
-SynapticsDetectFinger(SynapticsPrivatePtr priv, struct SynapticsHwState* hw)
+SynapticsDetectFinger(SynapticsPrivate *priv, struct SynapticsHwState *hw)
 {
-    SynapticsSHMPtr para = priv->synpara;
+    SynapticsSHM *para = priv->synpara;
     int finger;
 
     /* finger detection thru pressure and threshold */
@@ -736,7 +736,7 @@ SynapticsDetectFinger(SynapticsPrivatePtr priv, struct SynapticsHwState* hw)
 	priv->avg_width += (hw->fingerWidth - priv->avg_width + 1) / 2;
     if (finger && !priv->finger_flag) {
 	int safe_width = MAX(hw->fingerWidth, priv->avg_width);
-	if (hw->twoFingers || hw->threeFingers)
+	if (hw->numFingers > 1)
 	    finger = TRUE;			/* more than one finger -> not a palm */
 	else if ((safe_width < 6) && (priv->prev_z < para->finger_high))
 	    finger = TRUE;			/* thin finger, distinct touch -> not a palm */
@@ -765,8 +765,8 @@ SynapticsDetectFinger(SynapticsPrivatePtr priv, struct SynapticsHwState* hw)
 static int
 HandleState(LocalDevicePtr local, struct SynapticsHwState* hw)
 {
-    SynapticsPrivatePtr priv = (SynapticsPrivatePtr) (local->private);
-    SynapticsSHMPtr para = priv->synpara;
+    SynapticsPrivate *priv = (SynapticsPrivate *) (local->private);
+    SynapticsSHM *para = priv->synpara;
     Bool finger;
     int dist, dx, dy, buttons, id;
     edge_type edge;
@@ -779,26 +779,13 @@ HandleState(LocalDevicePtr local, struct SynapticsHwState* hw)
     long timeleft;
     int i;
 
-    /* If touchpad is switched off, we skip the whole thing and return delay */
-    if (para->touchpad_off == TRUE)
-	return delay;
-
-    edge = edge_detection(priv, hw->x, hw->y);
-
     dx = dy = 0;
 
-    /* update finger position in shared memory */
+    /* update hardware state in shared memory */
     para->x = hw->x;
     para->y = hw->y;
     para->z = hw->z;
-    if (hw->oneFinger)
-	para->numFingers = 1;
-    else if (hw->twoFingers)
-	para->numFingers = 2;
-    else if (hw->threeFingers)
-	para->numFingers = 3;
-    else
-	para->numFingers = 0;
+    para->numFingers = hw->numFingers;
     para->fingerWidth = hw->fingerWidth;
     para->left = hw->left;
     para->right = hw->right;
@@ -807,6 +794,10 @@ HandleState(LocalDevicePtr local, struct SynapticsHwState* hw)
     for (i = 0; i < 8; i++)
 	para->multi[i] = hw->multi[i];
 
+    /* If touchpad is switched off, we skip the whole thing and return delay */
+    if (para->touchpad_off == TRUE)
+	return delay;
+
     /* Treat the first two multi buttons as up/down for now. */
     hw->up |= hw->multi[0];
     hw->down |= hw->multi[1];
@@ -831,6 +822,8 @@ HandleState(LocalDevicePtr local, struct SynapticsHwState* hw)
 	hw->up = hw->down = FALSE;
     }
 
+    edge = edge_detection(priv, hw->x, hw->y);
+
     finger = SynapticsDetectFinger(priv, hw);
 
     /* tap and drag detection */
@@ -913,9 +906,9 @@ HandleState(LocalDevicePtr local, struct SynapticsHwState* hw)
     if (finger &&			/* finger is on the surface */
 	(timeleft > 0)) {		/* tap time is not succeeded */
 	/* count fingers when reported */
-	if (hw->twoFingers && (priv->finger_count == 0))
+	if ((hw->numFingers == 2) && (priv->finger_count == 0))
 	    priv->finger_count = 2;
-	if (hw->threeFingers)
+	if (hw->numFingers == 3)
 	    priv->finger_count = 3;
     } else { /* reset finger counts */
 	priv->finger_count = 0;
@@ -1183,8 +1176,8 @@ ConvertProc (LocalDevicePtr local,
 static Bool
 QueryHardware (LocalDevicePtr local)
 {
-    SynapticsPrivatePtr priv = (SynapticsPrivatePtr) local->private;
-    SynapticsSHMPtr para = priv->synpara;
+    SynapticsPrivate *priv = (SynapticsPrivate *) local->private;
+    SynapticsSHM *para = priv->synpara;
     int retries;
     int mode;
 
@@ -1245,7 +1238,7 @@ QueryHardware (LocalDevicePtr local)
 }
 
 static Bool
-SynapticsGetHwState(LocalDevicePtr local, SynapticsPrivatePtr priv,
+SynapticsGetHwState(LocalDevicePtr local, SynapticsPrivate *priv,
 		    struct SynapticsHwState *hw)
 {
     if (priv->proto == SYN_PROTO_PSAUX) {
@@ -1258,7 +1251,7 @@ SynapticsGetHwState(LocalDevicePtr local, SynapticsPrivatePtr priv,
 }
 
 static Bool
-SynapticsParseEventData(LocalDevicePtr local, SynapticsPrivatePtr priv,
+SynapticsParseEventData(LocalDevicePtr local, SynapticsPrivate *priv,
 			struct SynapticsHwState *hw)
 {
     struct input_event ev;
@@ -1269,6 +1262,14 @@ SynapticsParseEventData(LocalDevicePtr local, SynapticsPrivatePtr priv,
 	case EV_SYN:
 	    switch (ev.code) {
 	    case SYN_REPORT:
+		if (priv->oneFinger)
+		    priv->hwState.numFingers = 1;
+		else if (priv->twoFingers)
+		    priv->hwState.numFingers = 2;
+		else if (priv->threeFingers)
+		    priv->hwState.numFingers = 3;
+		else
+		    priv->hwState.numFingers = 0;
 		*hw = priv->hwState;
 		return Success;
 	    }
@@ -1312,13 +1313,13 @@ SynapticsParseEventData(LocalDevicePtr local, SynapticsPrivatePtr priv,
 		priv->hwState.multi[7] = v;
 		break;
 	    case BTN_TOOL_FINGER:
-		priv->hwState.oneFinger = v;
+		priv->oneFinger = v;
 		break;
 	    case BTN_TOOL_DOUBLETAP:
-		priv->hwState.twoFingers = v;
+		priv->twoFingers = v;
 		break;
 	    case BTN_TOOL_TRIPLETAP:
-		priv->hwState.threeFingers = v;
+		priv->threeFingers = v;
 		break;
 	    }
 	    break;
@@ -1328,11 +1329,7 @@ SynapticsParseEventData(LocalDevicePtr local, SynapticsPrivatePtr priv,
 		priv->hwState.x = ev.value;
 		break;
 	    case ABS_Y:
-		if (priv->swapY) {
-		    priv->hwState.y = YMAX_NOMINAL + YMIN_NOMINAL - ev.value;
-		} else {
-		    priv->hwState.y = ev.value;
-		}
+		priv->hwState.y = ev.value;
 		break;
 	    case ABS_PRESSURE:
 		priv->hwState.z = ev.value;
@@ -1342,40 +1339,13 @@ SynapticsParseEventData(LocalDevicePtr local, SynapticsPrivatePtr priv,
 		break;
 	    }
 	    break;
-	case EV_MSC:
-	    switch (ev.code) {
-	    case MSC_GESTURE:
-		priv->swapY = TRUE;
-		switch (ev.value) {
-		case 0:
-		    priv->hwState.oneFinger = FALSE;
-		    priv->hwState.twoFingers = TRUE;
-		    priv->hwState.threeFingers = FALSE;
-		    priv->hwState.fingerWidth = 5;
-		    break;
-		case 1:
-		    priv->hwState.oneFinger = FALSE;
-		    priv->hwState.twoFingers = FALSE;
-		    priv->hwState.threeFingers = TRUE;
-		    priv->hwState.fingerWidth = 5;
-		    break;
-		default:
-		    priv->hwState.oneFinger = TRUE;
-		    priv->hwState.twoFingers = FALSE;
-		    priv->hwState.threeFingers = FALSE;
-		    priv->hwState.fingerWidth = ev.value;
-		    break;
-		}
-		break;
-	    }
-	    break;
 	}
     }
     return !Success;
 }
 
 static Bool
-SynapticsReadEvent(SynapticsPrivatePtr priv, struct input_event *ev)
+SynapticsReadEvent(SynapticsPrivate *priv, struct input_event *ev)
 {
     int i, c;
     unsigned char *pBuf, u;
@@ -1391,7 +1361,7 @@ SynapticsReadEvent(SynapticsPrivatePtr priv, struct input_event *ev)
 }
 
 static Bool
-SynapticsParseRawPacket(LocalDevicePtr local, SynapticsPrivatePtr priv,
+SynapticsParseRawPacket(LocalDevicePtr local, SynapticsPrivate *priv,
 			struct SynapticsHwState *hw)
 {
     Bool ret = SynapticsGetPacket(local, priv);
@@ -1470,9 +1440,7 @@ SynapticsParseRawPacket(LocalDevicePtr local, SynapticsPrivatePtr priv,
 
     hw->y = YMAX_NOMINAL + YMIN_NOMINAL - hw->y;
 
-    hw->oneFinger = FALSE;
-    hw->twoFingers = FALSE;
-    hw->threeFingers = FALSE;
+    hw->numFingers = 0;
     hw->fingerWidth = 0;
 
     if (hw->z > 0) {
@@ -1496,15 +1464,15 @@ SynapticsParseRawPacket(LocalDevicePtr local, SynapticsPrivatePtr priv,
 
 	switch (w) {
 	case 0:
-	    hw->twoFingers = TRUE;
+	    hw->numFingers = 2;
 	    hw->fingerWidth = 5;
 	    break;
 	case 1:
-	    hw->threeFingers = TRUE;
+	    hw->numFingers = 3;
 	    hw->fingerWidth = 5;
 	    break;
 	default:
-	    hw->oneFinger = TRUE;
+	    hw->numFingers = 1;
 	    hw->fingerWidth = w;
 	    break;
 	}
@@ -1519,12 +1487,12 @@ SynapticsParseRawPacket(LocalDevicePtr local, SynapticsPrivatePtr priv,
  * Decide if the current packet stored in priv->protoBuf is valid.
  */
 static Bool
-PacketOk(SynapticsPrivatePtr priv)
+PacketOk(SynapticsPrivate *priv)
 {
     unsigned char *buf = priv->protoBuf;
     int newabs = SYN_MODEL_NEWABS(priv->model_id);
 
-    if (newabs ? ((buf[0] & 0xC8) != 0x80) : ((buf[0] & 0xC0) != 0xC0)) {
+    if (newabs ? ((buf[0] & 0xC0) != 0x80) : ((buf[0] & 0xC0) != 0xC0)) {
 	DBG(4, ErrorF("Synaptics driver lost sync at 1st byte\n"));
 	return FALSE;
     }
@@ -1534,7 +1502,7 @@ PacketOk(SynapticsPrivatePtr priv)
 	return FALSE;
     }
 
-    if ((newabs ? ((buf[3] & 0xc8) != 0xc0) : ((buf[3] & 0xc0) != 0x80))) {
+    if ((newabs ? ((buf[3] & 0xC0) != 0xC0) : ((buf[3] & 0xC0) != 0x80))) {
 	DBG(4, ErrorF("Synaptics driver lost sync at 4th byte\n"));
 	return FALSE;
     }
@@ -1548,7 +1516,7 @@ PacketOk(SynapticsPrivatePtr priv)
 }
 
 static Bool
-SynapticsGetPacket(LocalDevicePtr local, SynapticsPrivatePtr priv)
+SynapticsGetPacket(LocalDevicePtr local, SynapticsPrivate *priv)
 {
     int count = 0;
     int c;
@@ -1615,7 +1583,7 @@ SynapticsGetPacket(LocalDevicePtr local, SynapticsPrivatePtr priv)
 }
 
 static void
-PrintIdent(SynapticsPrivatePtr priv)
+PrintIdent(SynapticsPrivate *priv)
 {
     xf86Msg(X_PROBED, " Synaptics Touchpad, model: %d\n", SYN_ID_MODEL(priv->identity));
     xf86Msg(X_PROBED, " Firmware: %d.%d\n", SYN_ID_MAJOR(priv->identity),
